#include "armachine.h"

/**
 * @brief 생성자
 */
ARMachine::ARMachine()
{
	systemMode = USER;
	for(int i=0; i<16; i++) {
		r[i] = i*0x111111;
		c[i] = i*359;
	}
	CPSR = 0xE0000000;
}

/**
 * @brief ARM 명령어 한 개를 실행한다.
 * @param inst ARM 명령어 객체
 */
void ARMachine::Execute(ARMinstruction inst)
{
	INCREASE_PC();
	switch(inst.type) {
		case MOV: _executeMov(inst); break;
		default:
			printf("알 수 없는 명령어: %d\n", inst.type);
	}
}

/**
  * @brief CPSR의 N플래그 설정
  * @param value 1 또는 0(true or false)
  */
void ARMachine::setNflag(bool value)
{
	if(value == true)
		CPSR |= 0x8000;
	else
		CPSR &= 0x7FFF;
}

bool ARMachine::zflag()
{	return (CPSR & 0x4000);		}

/**
  * @brief CPSR의 Z플래그 설정
  * @param value 1 또는 0(true or false)
  */
void ARMachine::setZflag(bool value)
{
	if(value == true)	CPSR |=  0x4000;
	else CPSR &= 0xBFFF;
}

bool ARMachine::cflag()
{	return (CPSR & 0x2000);		}

/**
  * @brief CPSR의 C플래그 설정
  * @param value 1 또는 0(true or false)
  */
void ARMachine::setCflag(bool value)
{
	if(value == true)	CPSR |=  0x2000;
	else CPSR &= 0xD000;
}

/**
  @brief ARM 매뉴얼의 ConditionPassed method 구현
  @param cond 명령어의 상태 비트
  @return 명령어의 상태 요구 정보와 현재 CPSR의 상태 비트가 일치하면 true, 아니면 false
  */
bool ARMachine::_conditionPassed(uint8_t cond)
{
	return ( (CPSR>>28) & 0xF ) == cond ? true : false;
}

/**
  @brief 현재 시스템 모드가 SPSR을 가지고 있는가 식별
  */
bool ARMachine::_currentModeHasSPSR()
{
	if(systemMode == USER || systemMode == SYSTEM)
		return false;
	else
		return true;
}
