#ifndef ARMACHINE_H
#define ARMACHINE_H

#include<stdint.h>
#include<vector>
#include "arminstruction.h"
#include "mimicexception.h"
#include "misc.h"

using std::vector;

/**
  * @brief ARM 가상머신 클래스
  */
class ARMachine
{
public:
	ARMachine();
	/// 범용 레지스터
	uint32_t r[16];
	/// CP15 레지스터
	uint32_t c[16];
	uint32_t /** 상태 레지스터 */ CPSR, /** 저장된 상태 레지스터 */ SPSR;
	/// 실행 코드
	// vector<ARMinstruction> codes;
	/// 시스템 모드
	uint8_t systemMode;

	// 함수들
	void Execute(ARMinstruction inst);

	void setGPR(uint8_t reg_num, uint32_t value);
	bool zflag();
	bool cflag();
	void setNflag(bool value);
	void setZflag(bool value);
	void setCflag(bool value);

private:
	// 외부에 노출될 일 없는 함수들
	void _executeMov(ARMinstruction inst);

	bool _conditionPassed(uint8_t cond);
	bool _currentModeHasSPSR();

	// PC를 증가시킨다.
	void inline INCREASE_PC() { r[15] += 4; }
};

enum mode {
	NONE,
	USER,
	SYSTEM,
};

#endif // ARMACHINE_H
